@@ -1,4 +1,20 @@
-# encoding: utf-8
+# Licensed to Elasticsearch B.V. under one or more contributor
+# license agreements. See the NOTICE file distributed with
+# this work for additional information regarding copyright
+# ownership. Elasticsearch B.V. licenses this file to you under
+# the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#  http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 require "pluginmanager/command"
 require "pluginmanager/templates/render_context"
 require "erb"
@@ -76,6 +92,7 @@ class LogStash::PluginManager::Generate < LogStash::PluginManager::Command
       :author => git_data.author,
       :email  => git_data.email,
       :min_version => "2.0",
+      :logstash_path => File.expand_path('../..', File.dirname(__FILE__))
     }
   end
 
