@@ -1,9 +1,25 @@
-# encoding: utf-8
+# Licensed to Elasticsearch B.V. under one or more contributor
+# license agreements. See the NOTICE file distributed with
+# this work for additional information regarding copyright
+# ownership. Elasticsearch B.V. licenses this file to you under
+# the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#  http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 require_relative "pack_command"
 
 class LogStash::PluginManager::Pack < LogStash::PluginManager::PackCommand
-  option "--tgz", :flag, "compress package as a tar.gz file", :default => !LogStash::Environment.windows?
-  option "--zip", :flag, "compress package as a zip file", :default => LogStash::Environment.windows?
+  option "--tgz", :flag, "compress package as a tar.gz file", :default => !LogStash::Environment.win_platform?
+  option "--zip", :flag, "compress package as a zip file", :default => LogStash::Environment.win_platform?
   option "--[no-]clean", :flag, "clean up the generated dump of plugins", :default => true
   option "--overwrite", :flag, "Overwrite a previously generated package file", :default => false
 
