@@ -49,6 +49,7 @@ LogStash::Environment.load_locale!
 module LogStash::Config::Mixin
 
   include LogStash::Util::SubstitutionVariables
+  include LogStash::Util::Loggable
 
   attr_accessor :config
   attr_accessor :original_params
@@ -99,6 +100,17 @@ module LogStash::Config::Mixin
       params[name.to_s] = deep_replace(value)
     end
 
+    # Intercept codecs that have not been instantiated
+    params.each do |name, value|
+      validator = self.class.validator_find(name)
+      next unless validator && validator[:validate] == :codec && value.kind_of?(String)
+
+      codec_klass = LogStash::Plugin.lookup("codec", value)
+      codec_instance = LogStash::Plugins::Contextualizer.initialize_plugin(execution_context, codec_klass)
+
+      params[name.to_s] = LogStash::Codecs::Delegator.new(codec_instance)
+    end
+
     if !self.class.validate(params)
       raise LogStash::ConfigurationError,
         I18n.t("logstash.runner.configuration.invalid_plugin_settings")
@@ -183,6 +195,15 @@ module LogStash::Config::Mixin
     end
 
     # Define a new configuration setting
+    #
+    # @param name [String, Symbol, Regexp]
+    # @param opts [Hash]: the options for this config parameter
+    # @option opts [Array,Symbol] :validate
+    #   When `Array`, the expanded form of the given directive MUST exist in the Array.
+    #   When `Symbol`, the named validator matching the provided `Symbol` is used.
+    # @option opts [Boolean]      :list
+    # @option opts [Object]       :default
+    # @option opts [Boolean]      :required
     def config(name, opts={})
       @config ||= Hash.new
       # TODO(sissel): verify 'name' is of type String, Symbol, or Regexp
@@ -190,7 +211,7 @@ module LogStash::Config::Mixin
       name = name.to_s if name.is_a?(Symbol)
       @config[name] = opts  # ok if this is empty
 
-      if name.is_a?(String)
+      if name.is_a?(String) && opts.fetch(:attr_accessor, true)
         define_method(name) { instance_variable_get("@#{name}") }
         define_method("#{name}=") { |v| instance_variable_set("@#{name}", v) }
       end
@@ -400,6 +421,32 @@ module LogStash::Config::Mixin
       return nil
     end
 
+    ##
+    # Performs deep replacement of the provided value, then performs validation and coercion.
+    #
+    # The provided validator can be nil, an Array of acceptable values, or a Symbol
+    # representing a named validator, and is the result of a configuration parameter's `:validate` option (@see DSL#config)
+    #
+    # @overload validate_value(value, validator)
+    #   Validation occurs with the named validator.
+    #   @param value [Object]
+    #   @param validator [Symbol]
+    # @overload validate_value(value, validator)
+    #   The value must exist in the provided Array.
+    #   @param value [Object]
+    #   @param validator [Array]
+    # @overload validate_value(value, validator)
+    #   The value is always considered valid
+    #   @param value [Object]
+    #   @param validator [nil]
+    #
+    # @return [Array<(true, Object)>]: when value is valid, a tuple containing true and a coerced form of the value is returned
+    # @return [Array<(false, String)>]: when value is not valid, a tuple containing false and an error string is returned.
+    #
+    # @api private
+    #
+    # WARNING: validators added here must be back-ported to the Validation Support plugin mixin so that plugins
+    #          that use them are not constrained to the version of Logstash that introduced the validator.
     def validate_value(value, validator)
       # Validator comes from the 'config' pieces of plugins.
       # They look like this
@@ -429,6 +476,11 @@ module LogStash::Config::Mixin
         case validator
           when :codec
             if value.first.is_a?(String)
+              # A plugin's codecs should be instantiated by `PluginFactory` or in `Config::Mixin#config_init(Hash)`,
+              # which ensure the inner plugin has access to the outer's execution context and metric store.
+              # This deprecation exists to warn plugins that call `Config::Mixin::validate_value` directly.
+              self.deprecation_logger.deprecated("Codec instantiated by `Config::Mixin::DSL::validate_value(String, :codec)` which cannot propagate parent plugin's execution context or metrics. ",
+                                                 self.logger.debug? ? {:backtrace => caller} : {})
               value = LogStash::Codecs::Delegator.new LogStash::Plugin.lookup("codec", value.first).new
               return true, value
             else
@@ -561,6 +613,24 @@ module LogStash::Config::Mixin
             rescue ArgumentError
               return false, "Unparseable filesize: #{value.first}. possible units (KiB, MiB, ...) e.g. '10 KiB'. doc reference: http://www.elastic.co/guide/en/logstash/current/configuration.html#bytes"
             end
+          when :field_reference # @since 7.11
+            return [false, "Expected exactly one field reference, got `#{value.inspect}`"] unless value.kind_of?(Array) && value.size <= 1
+            return [true, nil] if value.empty? || value.first.nil? || value.first.empty?
+
+            candidate = value.first
+
+            return [false, "Expected a valid field reference, got `#{candidate.inspect}`"] unless org.logstash.FieldReference.isValid(candidate)
+
+            return [true, candidate]
+          when :sha_256_hex # since 8.3
+            return [false, "Expected exactly one hex-encoded SHA-256 fingerprint, got `#{value.inspect}`"] unless value.kind_of?(Array) && value.size <= 1
+            return [true, nil] if value.empty? || value.first.nil? || value.first.empty?
+
+            candidate = value.first
+
+            return [false, "Expected a hex-encoded SHA-256 fingerprint, got `#{candidate.inspect}`"] unless candidate.kind_of?(String) && candidate =~ /\A(?:[[:xdigit:]]{2}:?){32}\z/
+
+            return [true, candidate.upcase.tr('^0-9A-F','')]
           else
             return false, "Unknown validator symbol #{validator}"
         end # case validator
