@@ -24,20 +24,24 @@ require "logstash/outputs/base"
 require "logstash/instrument/collector"
 require "logstash/compiler"
 require "logstash/config/lir_serializer"
+require "logstash/worker_loop_thread"
 
-module LogStash; class JavaPipeline < JavaBasePipeline
+module LogStash; class JavaPipeline < AbstractPipeline
   include LogStash::Util::Loggable
 
   java_import org.apache.logging.log4j.ThreadContext
 
   attr_reader \
     :worker_threads,
+    :input_threads,
     :events_consumed,
     :events_filtered,
     :started_at,
     :thread
 
   MAX_INFLIGHT_WARN_THRESHOLD = 10_000
+  SECOND = 1
+  MEMORY = "memory".freeze
 
   def initialize(pipeline_config, namespaced_metric = nil, agent = nil)
     @logger = self.logger
@@ -46,7 +50,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
 
     @worker_threads = []
 
-    @drain_queue =  settings.get_value("queue.drain") || settings.get("queue.type") == "memory"
+    @drain_queue =  settings.get_value("queue.drain") || settings.get("queue.type") == MEMORY
 
     @events_filtered = java.util.concurrent.atomic.LongAdder.new
     @events_consumed = java.util.concurrent.atomic.LongAdder.new
@@ -69,6 +73,10 @@ module LogStash; class JavaPipeline < JavaBasePipeline
     # is by design and necessary for the wait_until_started semantic
     @finished_run = Concurrent::AtomicBoolean.new(false)
 
+    @logger.info(I18n.t('logstash.pipeline.effective_ecs_compatibility',
+                        :pipeline_id       => pipeline_id,
+                        :ecs_compatibility => settings.get('pipeline.ecs_compatibility')))
+
     @thread = nil
   end # def initialize
 
@@ -113,6 +121,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
     # this is useful in the context of pipeline reloading
     collect_stats
     collect_dlq_stats
+    initialize_flow_metrics
 
     @logger.debug("Starting pipeline", default_logging_keys)
 
@@ -134,9 +143,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
         run
         @finished_run.make_true
       rescue => e
-        # no need to log at ERROR level since this log will be redundant to the log in
-        # the worker loop thread global rescue clause
-        logger.debug("Pipeline terminated by worker error", error_log_params.call(e))
+        logger.error("Pipeline error", error_log_params.call(e))
       ensure
         # we must trap any exception here to make sure the following @finished_execution
         # is always set to true regardless of any exception before in the close method call
@@ -146,6 +153,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
           logger.error("Pipeline close error, ignoring", error_log_params.call(e))
         end
         @finished_execution.make_true
+        @logger.info("Pipeline terminated", "pipeline.id" => pipeline_id)
       end
     end
 
@@ -286,7 +294,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
       # Once all WorkerLoop have been initialized run them in separate threads
 
       worker_loops.each_with_index do |worker_loop, t|
-        thread = Thread.new do
+        thread = WorkerLoopThread.new(worker_loop) do
           Util.set_thread_name("[#{pipeline_id}]>worker#{t}")
           ThreadContext.put("pipeline.id", pipeline_id)
           begin
@@ -331,13 +339,38 @@ module LogStash; class JavaPipeline < JavaBasePipeline
   def monitor_inputs_and_workers
     twait = ThreadsWait.new(*(@input_threads + @worker_threads))
 
-    while !@input_threads.empty?
+    loop do
+      break if @input_threads.empty?
+
       terminated_thread = twait.next_wait
+
       if @input_threads.delete(terminated_thread).nil?
-        # this is a worker thread termination
-        # delete it from @worker_threads so that wait_for_workers does not wait for it
+        # this is an abnormal worker thread termination, we need to terminate the pipeline
+
         @worker_threads.delete(terminated_thread)
-        raise("Worker thread terminated in pipeline.id: #{pipeline_id}")
+
+        # before terminating the pipeline we need to close the inputs
+        stop_inputs
+
+        # wait 10 seconds for all input threads to terminate
+        wait_input_threads_termination(10 * SECOND) do
+          @logger.warn("Waiting for input plugin to close", default_logging_keys)
+          sleep(1)
+        end
+
+        if inputs_running? && settings.get("queue.type") == MEMORY
+          # if there are still input threads alive they are probably blocked pushing on the memory queue
+          # because no worker is present to consume from the ArrayBlockingQueue
+          # if this is taking too long we have a problem
+          wait_input_threads_termination(10 * SECOND) do
+            dropped_batch = filter_queue_client.read_batch
+            @logger.error("Dropping events to unblock input plugin", default_logging_keys(:count => dropped_batch.filteredSize)) if dropped_batch.filteredSize > 0
+          end
+        end
+
+        raise("Unable to stop input plugin(s)") if inputs_running?
+
+        break
       end
     end
 
@@ -420,7 +453,6 @@ module LogStash; class JavaPipeline < JavaBasePipeline
     stop_inputs
     wait_for_shutdown
     clear_pipeline_metrics
-    @logger.info("Pipeline terminated", "pipeline.id" => pipeline_id)
   end # def shutdown
 
   def wait_for_shutdown
@@ -502,6 +534,7 @@ module LogStash; class JavaPipeline < JavaBasePipeline
       # we want to keep other metrics like reload counts and error messages
       collector.clear("stats/pipelines/#{pipeline_id}/plugins")
       collector.clear("stats/pipelines/#{pipeline_id}/events")
+      collector.clear("stats/pipelines/#{pipeline_id}/flow")
     end
   end
 
@@ -518,6 +551,14 @@ module LogStash; class JavaPipeline < JavaBasePipeline
     }
   end
 
+  def shutdown_requested?
+    @shutdownRequested.get
+  end
+
+  def worker_threads_draining?
+    @worker_threads.any? {|t| t.worker_loop.draining? }
+  end
+
   private
 
   def close_plugin_and_ignore(plugin)
@@ -581,4 +622,18 @@ module LogStash; class JavaPipeline < JavaBasePipeline
 
     false
   end
+
+  def wait_input_threads_termination(timeout_seconds, &block)
+    start = Time.now
+    seconds = 0
+    while inputs_running? && (seconds < timeout_seconds)
+      block.call
+      seconds = Time.now - start
+    end
+  end
+
+  def inputs_running?
+    @input_threads.any?(&:alive?)
+  end
+
 end; end
