@@ -16,16 +16,31 @@
 # under the License.
 
 require "fileutils"
+require "delegate"
+
 require "logstash/util/byte_value"
 require "logstash/util/substitution_variables"
 require "logstash/util/time_value"
+require "i18n"
 
 module LogStash
+
   class Settings
 
     include LogStash::Util::SubstitutionVariables
     include LogStash::Util::Loggable
 
+    # The `LOGGABLE_PROXY` is included into `LogStash::Setting` to make all
+    # settings-related logs and deprecations come from the same logger.
+    LOGGABLE_PROXY = Module.new do
+      define_method(:logger) { Settings.logger }
+      define_method(:deprecation_logger) { Settings.deprecation_logger }
+
+      def self.included(base)
+        base.extend(self)
+      end
+    end
+
     # there are settings that the pipeline uses and can be changed per pipeline instance
     PIPELINE_SETTINGS_WHITE_LIST = [
       "config.debug",
@@ -34,9 +49,10 @@ module LogStash
       "config.reload.interval",
       "config.string",
       "dead_letter_queue.enable",
+      "dead_letter_queue.flush_interval",
       "dead_letter_queue.max_bytes",
+      "dead_letter_queue.storage_policy",
       "metric.collect",
-      "pipeline.java_execution",
       "pipeline.plugin_classloaders",
       "path.config",
       "path.dead_letter_queue",
@@ -48,6 +64,7 @@ module LogStash
       "pipeline.system",
       "pipeline.workers",
       "pipeline.ordered",
+      "pipeline.ecs_compatibility",
       "queue.checkpoint.acks",
       "queue.checkpoint.interval",
       "queue.checkpoint.writes",
@@ -70,6 +87,8 @@ module LogStash
     end
 
     def register(setting)
+      return setting.map { |s| register(s) } if setting.kind_of?(Array)
+
       if @settings.key?(setting.name)
         raise ArgumentError.new("Setting \"#{setting.name}\" has already been registered as #{setting.inspect}")
       else
@@ -83,7 +102,7 @@ module LogStash
 
     def get_setting(setting_name)
       setting = @settings[setting_name]
-      raise ArgumentError.new("Setting \"#{setting_name}\" hasn't been registered") if setting.nil?
+      raise ArgumentError.new("Setting \"#{setting_name}\" doesn't exist. Please check if you haven't made a typo.") if setting.nil?
       setting
     end
 
@@ -97,6 +116,10 @@ module LogStash
       settings
     end
 
+    def names
+      @settings.keys
+    end
+
     def set?(setting_name)
       get_setting(setting_name).set?
     end
@@ -129,6 +152,7 @@ module LogStash
     def to_hash
       hash = {}
       @settings.each do |name, setting|
+        next if setting.kind_of? Setting::DeprecatedAlias
         hash[name] = setting.value
       end
       hash
@@ -152,17 +176,7 @@ module LogStash
       output = []
       output << "-------- Logstash Settings (* means modified) ---------"
       @settings.each do |setting_name, setting|
-        value = setting.value
-        default_value = setting.default
-        if default_value == value # print setting and its default value
-          output << "#{setting_name}: #{value.inspect}" unless value.nil?
-        elsif default_value.nil? # print setting and warn it has been set
-          output << "*#{setting_name}: #{value.inspect}"
-        elsif value.nil? # default setting not set by user
-          output << "#{setting_name}: #{default_value.inspect}"
-        else # print setting, warn it has been set, and show default value
-          output << "*#{setting_name}: #{value.inspect} (default: #{default_value.inspect})"
-        end
+        setting.format(output)
       end
       output << "--------------- Logstash Settings -------------------"
       output
@@ -222,7 +236,7 @@ module LogStash
   end
 
   class Setting
-    include LogStash::Util::Loggable
+    include LogStash::Settings::LOGGABLE_PROXY
 
     attr_reader :name, :default
 
@@ -280,6 +294,10 @@ module LogStash
       }
     end
 
+    def inspect
+      "<#{self.class.name}(#{name}): #{value.inspect}" + (@value_is_set ? '' : ' (DEFAULT)') + ">"
+    end
+
     def ==(other)
       self.to_hash == other.to_hash
     end
@@ -288,6 +306,32 @@ module LogStash
       validate(value)
     end
 
+    def with_deprecated_alias(deprecated_alias_name)
+      SettingWithDeprecatedAlias.wrap(self, deprecated_alias_name)
+    end
+
+    ##
+    # Returns a Nullable-wrapped self, effectively making the Setting optional.
+    def nullable
+      Nullable.new(self)
+    end
+
+    def format(output)
+      effective_value = self.value
+      default_value = self.default
+      setting_name = self.name
+
+      if default_value == value # print setting and its default value
+        output << "#{setting_name}: #{effective_value.inspect}" unless effective_value.nil?
+      elsif default_value.nil? # print setting and warn it has been set
+        output << "*#{setting_name}: #{effective_value.inspect}"
+      elsif effective_value.nil? # default setting not set by user
+        output << "#{setting_name}: #{default_value.inspect}"
+      else # print setting, warn it has been set, and show default value
+        output << "*#{setting_name}: #{effective_value.inspect} (default: #{default_value.inspect})"
+      end
+    end
+
     protected
     def validate(input)
       if !input.is_a?(@klass)
@@ -481,6 +525,69 @@ module LogStash
       end
     end
 
+    class Password < Coercible
+      def initialize(name, default=nil, strict=true)
+        super(name, LogStash::Util::Password, default, strict)
+      end
+
+      def coerce(value)
+        return value if value.kind_of?(LogStash::Util::Password)
+
+        if value && !value.kind_of?(::String)
+          raise(ArgumentError, "Setting `#{name}` could not coerce non-string value to password")
+        end
+
+        LogStash::Util::Password.new(value)
+      end
+
+      def validate(value)
+        super(value)
+      end
+    end
+
+    class ValidatedPassword < Setting::Password
+      def initialize(name, value, password_policies)
+        @password_policies = password_policies
+        super(name, value, true)
+      end
+
+      def coerce(password)
+        if password && !password.kind_of?(::LogStash::Util::Password)
+          raise(ArgumentError, "Setting `#{name}` could not coerce LogStash::Util::Password value to password")
+        end
+
+        policies = build_password_policies
+        validatedResult = LogStash::Util::PasswordValidator.new(policies).validate(password.value)
+        if validatedResult.length() > 0
+          if @password_policies.fetch(:mode).eql?("WARN")
+            logger.warn("Password #{validatedResult}.")
+          else
+            raise(ArgumentError, "Password #{validatedResult}.")
+          end
+        end
+        password
+      end
+
+      def build_password_policies
+        policies = {}
+        policies[Util::PasswordPolicyType::EMPTY_STRING] = Util::PasswordPolicyParam.new
+        policies[Util::PasswordPolicyType::LENGTH] = Util::PasswordPolicyParam.new("MINIMUM_LENGTH", @password_policies.dig(:length, :minimum).to_s)
+        if @password_policies.dig(:include, :upper).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::UPPER_CASE] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :lower).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::LOWER_CASE] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :digit).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::DIGIT] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :symbol).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::SYMBOL] = Util::PasswordPolicyParam.new
+        end
+        policies
+      end
+    end
+
     # The CoercibleString allows user to enter any value which coerces to a String.
     # For example for true/false booleans; if the possible_strings are ["foo", "true", "false"]
     # then these options in the config file or command line will be all valid: "foo", true, false, "true", "false"
@@ -677,27 +784,122 @@ module LogStash
       end
     end
 
-    ##
-    # Instances of `DeprecatedSetting` can be registered, but will fail with helpful guidance when encountering any
-    # configuration that attempts to explicitly set the value. They should be used in the Major version immediately
-    # following a deprecation to assist users who are porting forward configurations.
-    class DeprecatedSetting < Setting
-      def initialize(name, guidance='please remove the setting from your configuration and try again.')
-        super(name, Object)
-        @guidance = guidance
+    # @see Setting#nullable
+    # @api internal
+    class Nullable < SimpleDelegator
+      def validate(value)
+        return true if value.nil?
+
+        __getobj__.send(:validate, value)
       end
 
-      def set(value)
-        fail(ArgumentError, "The setting `#{name}` has been deprecated and removed from Logstash; #{@guidance}")
+      # prevent delegate from intercepting
+      def validate_value
+        validate(value)
       end
     end
 
-    # Useful when a setting has been renamed but otherwise is semantically identical
-    class DeprecatedAndRenamed < DeprecatedSetting
-      attr_reader :new_name
-      def initialize(name, new_name)
-        super(name, "please update your configuration to use `#{new_name}` instead.")
-        @new_name = new_name
+    ##
+    # @api private
+    #
+    # A DeprecatedAlias provides a deprecated alias for a setting, and is meant
+    # to be used exclusively through `SettingWithDeprecatedAlias#wrap`
+    class DeprecatedAlias < SimpleDelegator
+      # include LogStash::Util::Loggable
+      alias_method :wrapped, :__getobj__
+      attr_reader :canonical_proxy
+
+      def initialize(canonical_proxy, alias_name)
+        @canonical_proxy = canonical_proxy
+
+        clone = @canonical_proxy.canonical_setting.clone
+        clone.instance_variable_set(:@name, alias_name)
+        clone.instance_variable_set(:@default, nil)
+
+        super(clone)
+      end
+
+      def set(value)
+        deprecation_logger.deprecated(I18n.t("logstash.settings.deprecation.set",
+                                             deprecated_alias: name,
+                                             canonical_name: canonical_proxy.name))
+        super
+      end
+
+      def value
+        logger.warn(I18n.t("logstash.settings.deprecation.queried",
+                           deprecated_alias: name,
+                           canonical_name: canonical_proxy.name))
+        @canonical_proxy.value
+      end
+
+      def validate_value
+        # bypass deprecation warning
+        wrapped.validate_value if set?
+      end
+    end
+
+    ##
+    # A SettingWithDeprecatedAlias wraps any Setting to provide a deprecated
+    # alias, and hooks `Setting#validate_value` to ensure that a deprecation
+    # warning is fired when the setting is provided by its deprecated alias,
+    # or to produce an error when both the canonical name and deprecated
+    # alias are used together.
+    class SettingWithDeprecatedAlias < SimpleDelegator
+
+      ##
+      # Wraps the provided setting, returning a pair of connected settings
+      # including the canonical setting and a deprecated alias.
+      # @param canonical_setting [Setting]: the setting to wrap
+      # @param deprecated_alias_name [String]: the name for the deprecated alias
+      #
+      # @return [SettingWithDeprecatedAlias,DeprecatedSetting]
+      def self.wrap(canonical_setting, deprecated_alias_name)
+        setting_proxy = new(canonical_setting, deprecated_alias_name)
+
+        [setting_proxy, setting_proxy.deprecated_alias]
+      end
+
+      attr_reader :deprecated_alias
+      alias_method :canonical_setting, :__getobj__
+
+      def initialize(canonical_setting, deprecated_alias_name)
+        super(canonical_setting)
+
+        @deprecated_alias = DeprecatedAlias.new(self, deprecated_alias_name)
+      end
+
+      def set(value)
+        canonical_setting.set(value)
+      end
+
+      def value
+        return super if canonical_setting.set?
+
+        # bypass warning by querying the wrapped setting's value
+        return deprecated_alias.wrapped.value if deprecated_alias.set?
+
+        default
+      end
+
+      def set?
+        canonical_setting.set? || deprecated_alias.set?
+      end
+
+      def format(output)
+        return super unless deprecated_alias.set? && !canonical_setting.set?
+
+        output << "*#{self.name}: #{value.inspect} (via deprecated `#{deprecated_alias.name}`; default: #{default.inspect})"
+      end
+
+      def validate_value
+        if deprecated_alias.set? && canonical_setting.set?
+          fail(ArgumentError, I18n.t("logstash.settings.deprecation.ambiguous",
+                                     canonical_name: canonical_setting.name,
+                                     deprecated_alias: deprecated_alias.name))
+        end
+
+        super
       end
     end
   end
